#ifndef SELECT_AG
#define SELECT_AG 1

#include <algorithm>
#include <vector>
#include <iostream>
#include <assert.h>

namespace SELECT{

  //
  // Classifical pivot == this costs O(N)
  //
  template <typename T>
  int partition(std::vector<T> & a, size_t left, size_t right){

    //assert(left>=0);
    //assert(right<a.size());

    size_t p, j; 
    T pivot_value = a[right];        // take the pivot value

    p = left - 1; j = right;
    for (;;) {
      
      while(a[++p] < pivot_value);   // respect pivot invariant
      while(a[--j] > pivot_value);   // respect pivot invariant
      if (p>=j) break;               //  inversion  
      std::swap(a[p], a[j]);         // ripristinate pivot invariant    
    }
    std::swap(a[p], a[right]);

    return p;
  }
  //
  // Classical randomized Select == this costs O(N) 
  //
  template <typename T>
  T select_k(std::vector<T> & a, size_t left, size_t right, size_t k){

    size_t p, random;

    assert(k <= right);

    while(right>left) {              // while indexes are not swapped

      random = rand() % (right-left) + left;  // take a random one

      //std::cerr << " l=" << left << " r=" << right 
      //		<< " random=" << random << "  " ;
      std::swap(a[random], a[right]);  // and swap 

      p = partition(a, left, right); // partition

      //for (unsigned int i = 0; i < a.size(); i++)
      //std::cerr << a[i] << " ";
      //std::cerr << " p=" << p  << std::endl;
      
      if (k<p)                  // k is lt the rank of the pivot
	right = p - 1;
      else if (k>p)             // k is gt the rank of the pivot
	left = p + 1;
      else                      // k is the rank of the pivot
	return a[p];	
    }
    return a[right];            // in this case right == left
  };

};

#endif
